﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Jewles
{
     sealed class Game
     {
          public enum Statuses : byte
          {
               Playing = 1,
               TimeUp = 2,
               NoTurns = 3
          }

          public readonly GameField Field;
          public int Points { get; private set; }
          public Statuses Status { get; private set; } = Statuses.Playing;
          public bool Time => _end != -1;
          public TimeSpan RemainingTime => _end == -1 ? new TimeSpan() : TimeSpan.FromMilliseconds(_end - Environment.TickCount);
          private readonly int _end = -1;
          private readonly int _bgn;

          public Game(GameField gf)
          {
               Field = gf;
          }

          public Game(GameField gf, TimeSpan time)
          {
               Field = gf;
               _bgn = Environment.TickCount;
               _end = _bgn + (int)time.TotalMilliseconds;
          }

          public void CheckTime()
          {
               if (Time && _end < Environment.TickCount)
                    Status = Statuses.TimeUp;
          }

          public bool Turn(Point from, Point to)
          {
               Field.Switch(from, to);
               List<GemGroup> groups = new List<GemGroup>(Field.FindGroups());
               if (groups.Count == 0)
               {
                    Field.Switch(from, to);
                    return false;
               }


               do
               {
                    Points += GetPoints(groups.ToArray());
                    Field.RemoveGroups(groups.ToArray());
                    groups = new List<GemGroup>(Field.FindGroups());
               } while (groups.Count > 0);
               if (Field.GetHint().IsEmpty)
                    Status = Statuses.NoTurns;
               return true;
          }

          private int GetPoints(GemGroup[] groups)
          {
               Dictionary<Point, int> finds = new Dictionary<Point, int>();
               foreach (GemGroup g in groups)
                    foreach (Point p in g.Group)
                         if (!finds.ContainsKey(p))
                              finds.Add(p, 1);
                         else
                              finds[p]++;
               return finds.Sum(kvp => kvp.Value);
          }
     }
}
